####################################################################
## This code implements the SPEC method as described in 
## C. Bolen et al., BMC Bioinformatics 2011
## Author - Christopher Bolen   


#' The SPEC function
#' @description Runs SPEC on the list of query signatures given in query_list
#' @param exprsVals - a matrix of expression values (genes x patients)
#' @param queries - a list of gene sets to be run through SPEC
#' @param subsetEnrichment - a matrix of enrichment values generated by running "cellSubsetEnrich"
#' @return a matrix of correlations; rows represent cell subsets, and columns
#'         represent the query signatures
#' @export
SPEC = function(exprsVals, queries, subsetEnrichment){
  queryEnrichment = matrix( NA, nrow=length(queries), ncol=ncol(exprsVals),
                                        dimnames=list(queries,1:ncol(exprsVals)) )
  if(!is.list(queries)){
    queries = list(query=queries)
  }
  queryEnrichment = t(sapply(queries, function(q){
    signatureEnrichment(exprsVals,q)
  }))

  return(cor(t(subsetEnrichment),t(queryEnrichment)))
}

#' Calculate Cell Subset Enrichment
#' @description This function calculates the enrichment scores for the predefined cell subsets
#' @param exprsVals an expression matrix
#' @param subsets a list; entries are character vectors representing an individual
#'                cell subset. The default signatures are the IRIS signatures.
#' @return matrix of per-patient enrichment scores. Rows correspond to the subsets 
#' defined in 'subsets'
#' @export
cellSubsetEnrich = function(exprsVals, subsets=NULL){
  if(is.null(subsets)){
    cat("No cell subsets provided; Using the IRIS gene signatures.\n")
    subsets=irisSignatures
  }
  
  ## create matrix
  subsetEnrichment = t(sapply(subsets, function(subset){
     signatureEnrichment(exprsVals, subset)
  }))
  return(subsetEnrichment)
}

#' Calculate SPEC P-values
#' @description Calculates the P-values for a given correlation matrix
#' @param corMat - the corellation matrix from running SPEC
#' @param cellEnrichment - the cell subset enrichment scores from cellSubsetEnrich
#' @param randMatList - a list of the variable names for the matrices of random enrichment scores
#'             (split in parts because of size)
#' @param sizeLocs - which of the random matrices were generated with the same size as your query sigs 
#' @param s,g - the boolean vector of specific and general cell subset signatures for
#' @return matrix of p-values denoting the significance of the signature correlations
#' @export
calcPVals = function(corMat, cellEnrichment, randMatList, sizeLocs, 
                     s=rep(T, nrow(cellEnrichment)), g=!s){
  cors = NULL
  for(i in 1:length(randMatList)){
    cors = cbind(cors, cor(t(cellEnrichment), t(get(randMatList[i]))))
  }
  specMaxes = matrix(apply(cors[s,],2,max),nrow=1000)
  specMus = apply(specMaxes,2,mean)
  specSDs = apply(specMaxes,2,sd)
  
  genMaxes = matrix(apply(cors[g,],2,max),nrow=1000)
  genMus = apply(genMaxes,2,mean)
  genSDs = apply(genMaxes,2,sd)
  
  pVals = matrix(0, nrow(corMat), ncol(corMat), dimnames=dimnames(corMat))
  for(i in 1:length(sizeLocs)){
     for(j in 1:nrow(corMat)){
       if(s[j]){
         pVals[j,i] = 1-pnorm(corMat[j,i], specMus[sizeLocs[i]],specSDs[sizeLocs[i]])
       }else{
         pVals[j,i] = 1-pnorm(corMat[j,i], genMus[sizeLocs[i]],specSDs[sizeLocs[i]])
       }
     }
   }
   pVals     
}